@@ -2,8 +2,7 @@ use crate::config::{Alias, Config, PromptLevel};
 use crate::error::Result;
 use crate::list::Grouping;
 use crate::path;
-use crate::query::{EditQuery, ListQuery, RangeQuery, ScriptQuery};
-use crate::script::ScriptName;
+use crate::query::{EditQuery, ListQuery, RangeQuery, ScriptOrDirQuery, ScriptQuery};
 use crate::script_type::ScriptType;
 use crate::tag::TagFilter;
 use crate::Either;
@@ -19,7 +18,9 @@ mod completion;
 pub use completion::*;
 mod tags;
 pub use tags::*;
+mod help_str;
 mod types;
+use help_str::*;
 pub use types::*;
 
 const NO_FLAG_SETTINGS: &[AppSettings] = &[
@@ -37,12 +38,12 @@ pub struct RootArgs {
     pub hs_home: Option<String>,
     #[structopt(long, hidden = true)]
     pub dump_args: bool,
-    #[structopt(long, help = "Do not record history")]
+    #[structopt(long, help = "Don't record history")]
     pub no_trace: bool,
     #[structopt(
         long,
         conflicts_with = "no-trace",
-        help = "Do not affect script time (but will still record history)"
+        help = "Don't affect script time order (but still record history and affect time filter)"
     )]
     pub humble: bool,
     #[structopt(short = "A", long, global = true, help = "Show scripts NOT within recent days", conflicts_with_all = &["all", "timeless"])]
@@ -53,10 +54,9 @@ pub struct RootArgs {
         short,
         long,
         global = true,
-        parse(try_from_str),
         conflicts_with = "all",
         number_of_values = 1,
-        help = "Filter by tags, e.g. `all,^mytag`"
+        help = "Filter by tags, e.g. `all,^removed`"
     )]
     pub filter: Vec<TagFilter>,
     #[structopt(
@@ -162,7 +162,7 @@ pub enum Subs {
     Help { args: Vec<String> },
     #[structopt(setting = Hidden, about = "Print the help message of env variables")]
     EnvHelp {
-        #[structopt(default_value = "-", parse(try_from_str))]
+        #[structopt(default_value = "-")]
         script_query: ScriptQuery,
     },
     #[structopt(setting = Hidden)]
@@ -171,20 +171,15 @@ pub enum Subs {
     Migrate,
     #[structopt(about = "Edit hyper script", settings = &[AllowLeadingHyphen, TrailingVarArg])]
     Edit {
-        #[structopt(
-            long,
-            short = "T",
-            parse(try_from_str),
-            help = "Type of the script, e.g. `sh`"
-        )]
+        #[structopt(long, short = "T", help = TYPE_HELP)]
         ty: Option<ScriptType>,
         #[structopt(long, short)]
         no_template: bool,
-        #[structopt(long, short)]
+        #[structopt(long, short, help = TAGS_HELP)]
         tags: Option<TagFilter>,
         #[structopt(long, help = "Create script without invoking the editor")]
         fast: bool,
-        #[structopt(parse(try_from_str), default_value = "?")]
+        #[structopt(default_value = "?", help = NEW_ANONYMOUS_HELP)]
         edit_query: EditQuery<ScriptQuery>,
         content: Vec<String>,
     },
@@ -219,26 +214,26 @@ pub enum Subs {
             help = "Raise an error if --previous-args is given but there is no previous argument"
         )]
         error_no_previous: bool,
-        #[structopt(long, short, requires = "previous-args")]
+        #[structopt(long, short, requires = "previous-args", help = "")]
         dir: Option<PathBuf>,
-        #[structopt(default_value = "-", parse(try_from_str))]
+        #[structopt(default_value = "-")]
         script_query: ScriptQuery,
         #[structopt(help = "Command line args to pass to the script")]
         args: Vec<String>,
     },
     #[structopt(about = "Execute the script query and get the exact file")]
     Which {
-        #[structopt(default_value = "-", parse(try_from_str))]
+        #[structopt(default_value = "-")]
         script_query: ScriptQuery,
     },
     #[structopt(about = "Print the script to standard output")]
     Cat {
-        #[structopt(default_value = "-", parse(try_from_str))]
+        #[structopt(default_value = "-")]
         script_query: ScriptQuery,
     },
     #[structopt(about = "Remove the script")]
     RM {
-        #[structopt(parse(try_from_str), required = true, min_values = 1)]
+        #[structopt(required = true, min_values = 1)]
         queries: Vec<ListQuery>,
         #[structopt(
             long,
@@ -252,29 +247,23 @@ pub enum Subs {
     Types(Types),
     #[structopt(about = "Copy the script to another one")]
     CP {
-        #[structopt(long, short)]
+        #[structopt(long, short, help = TAGS_HELP)]
         tags: Option<TagFilter>,
         origin: ScriptQuery,
-        new: EditQuery<ScriptName>,
+        #[structopt(help = NEW_ANONYMOUS_HELP)]
+        new: EditQuery<ScriptOrDirQuery>,
     },
     #[structopt(about = "Move the script to another one")]
     MV {
-        #[structopt(
-            long,
-            short = "T",
-            parse(try_from_str),
-            help = "Type of the script, e.g. `sh`"
-        )]
+        #[structopt(long, short = "T", help = TYPE_HELP)]
         ty: Option<ScriptType>,
-        #[structopt(short, long)]
+        #[structopt(long, short, help = TAGS_HELP)]
         tags: Option<TagFilter>,
-        #[structopt(parse(try_from_str))]
         origin: ListQuery,
-        new: Option<EditQuery<ScriptName>>,
+        #[structopt(help = NEW_ANONYMOUS_HELP)]
+        new: Option<EditQuery<ScriptOrDirQuery>>,
     },
-    #[structopt(
-        about = "Manage script tags. If a tag filter is given, store it to config, otherwise show tag information."
-    )]
+    #[structopt(about = "Manage script tags")]
     Tags(Tags),
     #[structopt(about = "Manage script history")]
     History {
@@ -286,7 +275,8 @@ pub enum Subs {
 #[derive(StructOpt, Debug, Serialize)]
 pub enum History {
     RM {
-        #[structopt(parse(try_from_str), required = true, min_values = 1)]
+        // TODO: dir
+        #[structopt(required = true, min_values = 1)]
         queries: Vec<ListQuery>,
         range: RangeQuery,
     },
@@ -299,7 +289,7 @@ pub enum History {
     #[structopt(about = "Humble an event by it's id")]
     Humble { event_id: u64 },
     Show {
-        #[structopt(default_value = "-", parse(try_from_str))]
+        #[structopt(default_value = "-")]
         queries: Vec<ListQuery>,
         #[structopt(short, long, default_value = "10")]
         limit: u32,
@@ -311,7 +301,7 @@ pub enum History {
         dir: Option<PathBuf>,
     },
     Neglect {
-        #[structopt(parse(try_from_str), required = true, min_values = 1)]
+        #[structopt(required = true, min_values = 1)]
         queries: Vec<ListQuery>,
     },
     #[structopt( settings = NO_FLAG_SETTINGS)]
@@ -321,7 +311,7 @@ pub enum History {
         args: Vec<String>,
     },
     Tidy {
-        #[structopt(parse(try_from_str), required = true, min_values = 1)]
+        #[structopt(required = true, min_values = 1)]
         queries: Vec<ListQuery>,
     },
 }
@@ -340,7 +330,6 @@ pub struct List {
     pub file: bool,
     #[structopt(long, help = "Show name of the script.", conflicts_with = "long")]
     pub name: bool,
-    #[structopt(parse(try_from_str))]
     pub queries: Vec<ListQuery>,
 }
 
