@@ -7,7 +7,7 @@ use crate::tag::TagFilter;
 use crate::util::illegal_name;
 use chrono::NaiveDateTime;
 use fxhash::FxHashSet as HashSet;
-use serde::{Deserialize, Serialize};
+use serde::Serialize;
 use std::borrow::Cow;
 use std::cmp::Ordering;
 use std::ops::{Deref, DerefMut};
@@ -26,10 +26,50 @@ macro_rules! max {
     };
 }
 
-#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
+// XXX: use impl_ser_by_to_string
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deref, Display, Hash)]
+#[display(fmt = "{}", inner)]
+pub struct ConcreteScriptName {
+    #[deref]
+    inner: String,
+}
+impl ConcreteScriptName {
+    fn valid(s: &str) -> Result {
+        // FIXME: 好好想想什麼樣的腳本名可行，並補上單元測試
+        for s in s.split('/') {
+            if illegal_name(s) {
+                return Err(Error::Format(ScriptNameCode, s.to_owned()));
+            }
+        }
+        Ok(())
+    }
+    pub fn new(s: String) -> Result<Self> {
+        Self::valid(&s)?;
+        Ok(ConcreteScriptName { inner: s })
+    }
+    fn new_unchecked(s: String) -> Self {
+        ConcreteScriptName { inner: s }
+    }
+    fn stem_inner(&self) -> &str {
+        if let Some((_, stem)) = self.inner.rsplit_once('/') {
+            stem
+        } else {
+            &self.inner
+        }
+    }
+    pub fn stem(&self) -> ConcreteScriptName {
+        Self::new_unchecked(self.stem_inner().to_owned())
+    }
+    pub fn join(&mut self, other: &ConcreteScriptName) {
+        self.inner += "/";
+        self.inner += other.stem_inner();
+    }
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Hash)]
 pub enum ScriptName {
     Anonymous(u32),
-    Named(String),
+    Named(ConcreteScriptName),
 }
 impl FromStr for ScriptName {
     type Err = Error;
@@ -39,7 +79,12 @@ impl FromStr for ScriptName {
 }
 impl ScriptName {
     #[inline]
-    pub fn valid(mut s: &str, fuzzing: bool, check_legal_name: bool) -> Result<Option<u32>> {
+    pub fn valid(
+        mut s: &str,
+        allow_endwith_slash: bool,
+        allow_dot: bool,
+        check: bool,
+    ) -> Result<Option<u32>> {
         log::debug!("檢查腳本名：{}", s);
         let reg = regex::Regex::new(r"^\.(\d+)$")?;
         let m = reg.captures(s);
@@ -49,24 +94,21 @@ impl ScriptName {
                 Ok(id) => Ok(Some(id)),
                 Err(e) => Err(Error::Format(ScriptNameCode, s.to_owned())).context(e),
             }
-        } else {
-            if fuzzing {
-                if s == "." {
-                    log::info!("特殊規則：模糊搜時允許單一個`.`");
+        } else if check {
+            if s == "." {
+                if allow_dot {
+                    log::info!("特殊規則：允許單一個`.`");
                     return Ok(None); // NOTE: 讓匿名腳本可以直接用 `.` 來搜
-                } else if s.ends_with('/') {
-                    log::info!("特殊規則：模糊搜時允許以`/`結尾");
+                }
+            } else if s.ends_with('/') {
+                if allow_endwith_slash {
+                    log::info!("特殊規則：允許以`/`結尾");
                     s = &s[..s.len() - 1]; // NOTE: 有了補全，很容易補出帶著`/`結尾的指令，放寬標準吧
                 }
             }
-            // FIXME: 好好想想什麼樣的腳本名可行，並補上單元測試
-            if check_legal_name {
-                for s in s.split('/') {
-                    if illegal_name(s) {
-                        return Err(Error::Format(ScriptNameCode, s.to_owned()));
-                    }
-                }
-            }
+            ConcreteScriptName::valid(&s)?;
+            Ok(None)
+        } else {
             Ok(None)
         }
     }
@@ -135,7 +177,7 @@ impl ScriptName {
 impl PartialOrd for ScriptName {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         match (self, other) {
-            (ScriptName::Named(n1), ScriptName::Named(n2)) => n1.partial_cmp(n2),
+            (ScriptName::Named(n1), ScriptName::Named(n2)) => n1.inner.partial_cmp(&n2.inner),
             (ScriptName::Anonymous(i1), ScriptName::Anonymous(i2)) => i1.partial_cmp(i2),
             (ScriptName::Named(_), ScriptName::Anonymous(_)) => Some(Ordering::Less),
             (ScriptName::Anonymous(_), ScriptName::Named(_)) => Some(Ordering::Greater),
@@ -295,13 +337,18 @@ impl IntoScriptName for u32 {
         Ok(ScriptName::Anonymous(self))
     }
 }
+impl IntoScriptName for ConcreteScriptName {
+    fn into_script_name(self) -> Result<ScriptName> {
+        Ok(ScriptName::Named(self))
+    }
+}
 #[inline]
-fn string_into_script_name(s: String, check_legal_name: bool) -> Result<ScriptName> {
-    log::debug!("解析腳本名：{} {}", s, check_legal_name);
-    if let Some(id) = ScriptName::valid(&s, false, check_legal_name)? {
+fn string_into_script_name(s: String, check: bool) -> Result<ScriptName> {
+    log::debug!("解析腳本名：{} {}", s, check);
+    if let Some(id) = ScriptName::valid(&s, false, false, check)? {
         id.into_script_name()
     } else {
-        Ok(ScriptName::Named(s))
+        Ok(ScriptName::Named(ConcreteScriptName::new_unchecked(s))) // NOTE: already checked by `ScriptName::valid`
     }
 }
 impl IntoScriptName for String {
